package it.unibs.arnaldo.rovineperdute;

import java.util.*;

public class Graph {
    private final HashMap<RovinaPerduta, List<RovinaPerduta>> graph;

    public Graph(){
        graph = new HashMap<>();
    }

    public Graph(HashMap<RovinaPerduta, List<RovinaPerduta>> graph){
        this.graph = graph;
    }
    public void add(RovinaPerduta rp, List<RovinaPerduta> links) {
        graph.put(rp, links);
    }

    public HashMap<RovinaPerduta, List<RovinaPerduta>> getGraph() {
        return graph;
    }

    public List<RovinaPerduta> getListaCollegamenti(RovinaPerduta rp){
        return graph.get(rp);
    }

    @Override
    public String toString() {
        StringBuilder ret = new StringBuilder();
        for (RovinaPerduta rp : graph.keySet()){
            ret.append(rp.toString()).append(": \n");
            List<RovinaPerduta> rovineCollegate = graph.get(rp);
            for (RovinaPerduta rpCollegata: rovineCollegate){
                ret.append("\t").append(rpCollegata.toString()).append("\n");
            }
        }
        return ret.toString();
    }

    public List<RovinaPerdutaGrafo> calcolaSpanningTree(RovinaPerduta puntoPartenza, TipoVeicolo tipo){
        HashMap<RovinaPerduta, Double> distanze = new HashMap<>();
        HashMap<RovinaPerduta, RovinaPerduta> predecessori = new HashMap<>();
        HashMap<RovinaPerduta, Integer> salti = new HashMap<>();
        Queue<RovinaPerduta> coda = new PriorityQueue<>(Comparator.comparingDouble(distanze::get));

        // inizializzazione variabili
        for (RovinaPerduta rp : graph.keySet()) {
            distanze.put(rp, Double.POSITIVE_INFINITY);
            salti.put(rp, Integer.MAX_VALUE);
        }

        distanze.put(puntoPartenza, 0.0);
        coda.add(puntoPartenza);
        salti.put(puntoPartenza, 0);

        // fino a quando non ho visitato tutti i percorsi
        while (!coda.isEmpty()){
            RovinaPerduta rp = coda.poll();
            List<RovinaPerduta> listaVicine = graph.get(rp);

            for (RovinaPerduta vicina : listaVicine){
                double distanza = rp.getCoordinata().calcolaDistanza(vicina.getCoordinata(), tipo);
                double nuovaDistanza = distanze.get(rp) + distanza;
                double distanzaAttuale = distanze.get(vicina);
                int nuoviSalti = salti.get(rp) + 1;
                int saltiAttuali = salti.get(vicina);

                // aggiorno la distanza
                if (nuovaDistanza < distanzaAttuale
                        || (Double.compare(nuovaDistanza, distanzaAttuale) == 0 && nuoviSalti < saltiAttuali)
                        || (Double.compare(nuovaDistanza, distanzaAttuale) == 0 && nuoviSalti == saltiAttuali && rp.getId() > predecessori.getOrDefault(vicina, rp).getId())) {

                    distanze.put(vicina, nuovaDistanza);
                    predecessori.put(vicina, rp);
                    salti.put(vicina, nuoviSalti);
                    coda.add(vicina);
                }
            }
        }

        // trovo la rovina perduta
        RovinaPerduta rovinePerdute = null;
        int maxId = 0;

        for (RovinaPerduta rp : graph.keySet()) {
            if (rp.getId() > maxId) {
                maxId = rp.getId();
                rovinePerdute = rp;
            }
        }


        // ricostruisce il percorso dal punto di partenza alla rovina perduta
        LinkedList<RovinaPerdutaGrafo> percorso = new LinkedList<>();
        RovinaPerduta corrente = rovinePerdute;
        while (corrente != null) {
            percorso.addFirst(new RovinaPerdutaGrafo(corrente.getCoordinata(), corrente.getNome(), corrente.getId(), distanze.get(corrente)));
            corrente = predecessori.get(corrente);
        }

        return percorso;
    }
}
