package it.unibs.arnaldo.rovineperdute;

import javax.xml.stream.*;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class Parser {

    // rovina = città
    public static Graph parseRovinePerduteXML(String fileName) {

        XMLInputFactory xmlif;
        XMLStreamReader xmlr = null;
        String currentElement = null;

        // momentanea
        List<Integer> link = new ArrayList<>();

        // mappa id -> rovina
        HashMap<Integer, RovinaPerduta> rovineMap = new HashMap<>();

        // mappa id -> link collegamenti
        HashMap<Integer, List<Integer>> mappaCollegamenti = new HashMap<>();

        // variabili momentanee
        int cityId = -1;
        int x = -1;
        int y = -1;
        int h = -1;
        String nome = null;

        try (FileInputStream reader = new FileInputStream(fileName)) {
            xmlif = XMLInputFactory.newInstance();
            xmlr = xmlif.createXMLStreamReader(fileName, reader);

            while (xmlr.hasNext()) {
                int event = xmlr.getEventType();

                switch (event) {
                    case XMLStreamConstants.START_ELEMENT:
                        currentElement = xmlr.getLocalName();
                        if (currentElement.equals("city")) {
                            cityId = Integer.parseInt(xmlr.getAttributeValue(null, "id"));
                            nome = xmlr.getAttributeValue(null, "name");
                            x = Integer.parseInt(xmlr.getAttributeValue(null, "x"));
                            y = Integer.parseInt(xmlr.getAttributeValue(null, "y"));
                            h = Integer.parseInt(xmlr.getAttributeValue(null, "h"));
                        }
                        if (currentElement.equals("link")) {
                            link.add(Integer.parseInt(xmlr.getAttributeValue(null, "to")));
                        }
                        break;

                    case XMLStreamConstants.END_ELEMENT:
                        String endTag = xmlr.getLocalName();

                        if (endTag.equals("city")) {
                            mappaCollegamenti.put(cityId, link);
                            link = new ArrayList<>();
                            RovinaPerduta rovinaPerduta = new RovinaPerduta(new Coordinata(x, y, h), nome, cityId);
                            nome = null;
                            x = -1;
                            y = -1;
                            h = -1;

                            if (cityId != -1){
                                rovineMap.put(cityId, rovinaPerduta);
                            }

                            cityId = -1;
                        }

                        break;
                }

                xmlr.next();
            }

            xmlr.close();
            Graph grafo = new Graph();

            for (Integer idx : mappaCollegamenti.keySet()){
                List<Integer> listaIndiciCollegamenti = mappaCollegamenti.get(idx);
                List<RovinaPerduta> listaCollegamenti = new ArrayList<>();
                RovinaPerduta rp = rovineMap.get(idx);

                for(int i : listaIndiciCollegamenti){
                    listaCollegamenti.add(rovineMap.get(i));
                }

                grafo.add(rp, listaCollegamenti);
            }

            return grafo;

        } catch (FactoryConfigurationError | XMLStreamException | IOException e) {
            System.out.println("Error reading the file: " + e.getMessage());
            return null;
        }
    }

    public static void generaXML(List<RovinaPerdutaGrafo> minimumTreeTonatium, List<RovinaPerdutaGrafo> minimumTreeMetztli) {
        String fileName = "routes.xml";

        try (FileOutputStream outputStream = new FileOutputStream(fileName)) {
            XMLOutputFactory xmlOutputFactory = XMLOutputFactory.newInstance();
            XMLStreamWriter xmlWriter = xmlOutputFactory.createXMLStreamWriter(outputStream, "UTF-8");

            xmlWriter.writeStartDocument("UTF-8", "1.0");
            xmlWriter.writeStartElement("routes");

            // Tonatiuh
            double consumoTonatiuh = minimumTreeTonatium.get(minimumTreeTonatium.size() - 1).getDistanza();
            int numeroCittaTonatiuh = minimumTreeTonatium.size();

            xmlWriter.writeStartElement("route");
            xmlWriter.writeAttribute("team", "Tonatiuh");
            xmlWriter.writeAttribute("cost", String.format("%.0f", consumoTonatiuh));
            xmlWriter.writeAttribute("cities", String.valueOf(numeroCittaTonatiuh));

            for (RovinaPerdutaGrafo citta : minimumTreeTonatium) {
                xmlWriter.writeStartElement("city");
                xmlWriter.writeAttribute("id", String.valueOf(citta.getId()));
                xmlWriter.writeAttribute("name", citta.getNome());
                xmlWriter.writeEndElement(); // city
            }

            xmlWriter.writeEndElement(); // route

            // Metztli
            double consumoMetztli = minimumTreeMetztli.get(minimumTreeMetztli.size() - 1).getDistanza();
            int numeroCittaMetztli = minimumTreeMetztli.size();

            xmlWriter.writeStartElement("route");
            xmlWriter.writeAttribute("team", "Metztli");
            xmlWriter.writeAttribute("cost", String.format("%.0f", consumoMetztli));
            xmlWriter.writeAttribute("cities", String.valueOf(numeroCittaMetztli));

            for (RovinaPerdutaGrafo citta : minimumTreeMetztli) {
                xmlWriter.writeStartElement("city");
                xmlWriter.writeAttribute("id", String.valueOf(citta.getId()));
                xmlWriter.writeAttribute("name", citta.getNome());
                xmlWriter.writeEndElement(); // city
            }

            xmlWriter.writeEndElement();

            xmlWriter.writeEndElement();
            xmlWriter.writeEndDocument();
            xmlWriter.flush();

            System.out.println("File 'routes.xml' generato correttamente.");

        } catch (XMLStreamException | IOException e) {
            System.out.println("Errore durante la scrittura del file XML: " + e.getMessage());
        }
    }

}
